/*
 * Demonstrate the NSRL C++ class
 */

#include "config.h"

#include "bloom.h"
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <fcntl.h>
#include <openssl/sha.h>


int verify(nsrl_bloom *b)
{
    int errors = 0;
    int i;
    for(i=0;i<1000;i++){
	char number[20];
	u_char buf[20];
	
	memset(number,0,sizeof(number));
	sprintf(number,"%d",i);
	SHA1((u_char *)number,strlen(number),buf);
	int present = nsrl_bloom_query(b,buf);
	int even     = (i%2)==0;

	if(present != even){
	    printf("error: i=%d  present=%d even=%d\n",i,present,even);
	    errors++;
	}
    }
    return errors;
}

int main(int argc,char **argv)
{
    int errors =0;
    nsrl_bloom *b = nsrl_bloom_alloc();
    int i;

    nsrl_bloom_create(b,0,160,16,4,"A test bloom filter in memory");
    nsrl_bloom_print_info(b);

    printf("Storing SHA1 values of even numbers into bloom filter...\n");
    for(i=0;i<1000;i+=2){
	char number[20];
	u_char buf[20];
	memset(number,0,sizeof(number));
	sprintf(number,"%d",i);
	SHA1((u_char *)number,strlen(number),buf);
	nsrl_bloom_add(b,buf);
    }
    printf("Checking to make sure that odd numbers are not present and even numbers are present\n");
    errors += verify(b);

    /* Try writing it out to a file */
    nsrl_bloom_write(b,"/tmp/bloom.tmp");
    nsrl_bloom_free(b);

    printf("Opening /tmp/bloom.tmp\n");
    b = nsrl_bloom_alloc();
    if(nsrl_bloom_open(b,"/tmp/bloom.tmp",O_RDONLY)){
	fprintf(stderr,"Cannot open /tmp/bloom");
	exit(1);
    }
    nsrl_bloom_print_info(b);
    errors += verify(b);
    nsrl_bloom_free(b);
    return errors;
}
